import pytest
import pilot.client
from pilot.exc import PilotClientException
from unittest.mock import Mock
from gladier_tools.publish.publish import publish_gather_metadata


@pytest.fixture
def pilot_input():
    return {
        'dataset': 'my_dataset',
        'index': 'my_index',
        'project': 'my_project',
        'source_globus_endpoint': 'my_transfer_endpoint',
        'groups': []
    }


@pytest.fixture
def mock_pilot(monkeypatch):
    mock_inst = Mock()
    mock_client = Mock(return_value=mock_inst)
    mock_inst.get_globus_transfer_paths.return_value = [('src_path', 'dest_path')]
    monkeypatch.setattr(pilot.client, 'PilotClient', mock_client)
    return mock_client


def test_publish(pilot_input, mock_pilot):
    output = publish_gather_metadata(**pilot_input)
    assert 'search' in output
    assert 'transfer' in output


def test_publish_exception(pilot_input, mock_pilot):
    mock_pilot.side_effect = PilotClientException('Something bad happened!')
    output = publish_gather_metadata(**pilot_input)
    assert 'PilotClientException' in output
    assert 'Something bad happened!' in output



from gladier import GladierBaseTool


def untar_file(**data):
    import os
    import tarfile

    ##minimal data inputs payload
    file_path = data.get('file_path', '')
    file_name = data.get('file_name', '')
    output_path = data.get('output_path', '')
    ##

    full_path = os.path.join(file_path, file_name)

    if not os.path.isfile(full_path):
        raise NameError(f'{full_path}  does not exist!!')

    if not os.path.exists(output_path):
        os.makedirs(output_path)
    
    with tarfile.open(full_path) as file:
        file.extractall(output_path)
    return output_path

class UnTar(GladierBaseTool):
    """
    The UnTar tool makes it possible to extract data from Tar archives.

    :param file_path: Directory in which the tar file exists. Defaults to empty string if not passed.
    :param file_name: Name of the .tgz file which has to be untarred. Defaults to empty string if not passed.
    :param output_path: Location where the files will be extracted from the archive. Defaults to empty string if not passed.
    :param funcx_endpoint_non_compute: By default, uses the ``non-compute`` funcx endpoint.
    :returns output_path: Location of the extracted files.
    """

    flow_definition = {
        'Comment': 'UnTar a single file',
        'StartAt': 'UnTar',
        'States': {
            'UnTar': {
                'ActionUrl': 'https://api.funcx.org/automate',
                'Comment': None,
                'ExceptionOnActionFailure': True,
                'Parameters': {
                    'tasks': [
                        {
                            'endpoint.$': '$.input.funcx_endpoint_non_compute',
                            'func.$': '$.input.untar_file_funcx_id',
                            'payload.$': '$.input'
                        }
                    ]
                },
                'ResultPath': '$.UnTar',
                'Type': 'Action',
                'WaitTime': 300,
                'End': True,
            },
        }
    }

    funcx_functions = [untar_file]

    required_input = [
        'file_path',
        'file_name',
        'output_path',
        'funcx_endpoint_non_compute'
        ]




import os
from gladier_tools.posix.tar import tar


def test_tar_home_directory(mock_tar):
    output_file = tar(tar_input='~/foo')
    assert output_file == os.path.expanduser('~/foo.tgz')


def test_tar(mock_tar):
    mock_open, mock_context_manager = mock_tar
    output_file = tar(tar_input='foo')
    assert mock_open.called
    assert mock_context_manager.add.called
    assert output_file == 'foo.tgz'


def test_tar_trailing_slash(mock_tar):
    """This previously could result in /foo/.tgz"""
    output_file = tar(tar_input='/foo/')
    assert output_file == '/foo.tgz'



from gladier import GladierBaseTool, generate_flow_definition


def publish_gather_metadata(**data):
    import traceback
    from pilot.client import PilotClient
    from pilot.exc import PilotClientException, FileOrFolderDoesNotExist
    try:
        dataset, destination = data['dataset'], data.get('destination', '/')
        index, project, groups = data['index'], data['project'], data.get('groups', [])

        # Bootstrap Pilot
        pc = PilotClient(config_file=None, index_uuid=index)
        pc.project.set_project(project)
        # short_path is how pilot internally refers to datasets, implicitly accounting for
        # the endpoint and base project path. After publication, you may refer to your
        # dataset via the short path -- ``pilot describe short_path``
        short_path = pc.build_short_path(dataset, destination)
        return {
            'search': {
                'id': data.get('id', 'metadata'),
                'content': pc.gather_metadata(dataset, destination,
                                              custom_metadata=data.get('metadata')),
                'subject': pc.get_subject_url(short_path),
                'visible_to': [f'urn:globus:groups:id:{g}' for g in groups + [pc.get_group()]],
                'search_index': index
            },
            'transfer': {
                'source_endpoint_id': data['source_globus_endpoint'],
                'destination_endpoint_id': pc.get_endpoint(),
                'transfer_items': [{
                    'source_path': src,
                    'destination_path': dest,
                    # 'recursive': False,  # each file is explicit in pilot, no directories
                } for src, dest in pc.get_globus_transfer_paths(dataset, destination)]
            }
        }
    except (PilotClientException, FileOrFolderDoesNotExist):
        # FuncX does not allow for custom exceptions. Catch and print any pilot errors
        # so that FuncX does not encounter them.
        return traceback.format_exc()


class Publish(GladierBaseTool):
    """This function uses the globus-pilot tool to generate metadata compatible with
    portals on petreldata.net. Requires globus_pilot>=0.6.0.

    Publication happens in three steps:

    * PublishGatherMetadata -- A funcx function which uses globus-pilot to gather
      metadata on files or folders
    * PublishTransfer -- Transfers data to the Globus Endpoint selected in Globus Pilot
    * PublishIngest -- Ingest metadata gathered in fist step to Globus Search


    NOTE: This tool nests input under the 'pilot' keyword. Submit your input as the following:

    .. code-block::

        {
            'input': {
                'pilot': {
                    'dataset': 'foo',
                    'index': 'my-uuid'
                }
        }

    :param dataset: Path to file or directory
    :param destination: relative location under project directory to place dataset (Default `/`)
    :param source_globus_endpoint: The Globus Endpoint of the machine where you are executing
    :param index: The index to ingest this dataset in Globus Search
    :param project: The Pilot project to use for this dataset
    :param groups: A list of additional groups to make these records visible_to.

    Requires: the 'globus-pilot' package to be installed.
    """

    flow_definition = {
        'Comment': 'Publish metadata to Globus Search, with data from the result.',
        'StartAt': 'PublishGatherMetadata',
        'States': {
            'PublishGatherMetadata': {
                'Comment': 'Say something to start the conversation',
                'Type': 'Action',
                'ActionUrl': 'https://automate.funcx.org',
                'ActionScope': 'https://auth.globus.org/scopes/'
                               'b3db7e59-a6f1-4947-95c2-59d6b7a70f8c/action_all',
                'ExceptionOnActionFailure': False,
                'Parameters': {
                    'tasks': [{
                        'endpoint.$': '$.input.funcx_endpoint_non_compute',
                        'function.$': '$.input.publish_gather_metadata_funcx_id',
                        'payload.$': '$.input.pilot',
                    }]
                },
                'ResultPath': '$.PublishGatherMetadata',
                'WaitTime': 60,
                'Next': 'PublishTransfer',
            },
            'PublishTransfer': {
                'Comment': 'Transfer files for publication',
                'Type': 'Action',
                'ActionUrl': 'https://actions.automate.globus.org/transfer/transfer',
                'InputPath': '$.PublishGatherMetadata.details.result[0].transfer',
                'ResultPath': '$.PublishTransfer',
                'WaitTime': 600,
                'Next': 'PublishIngest',
            },
            'PublishIngest': {
                'Comment': 'Ingest the search document',
                'Type': 'Action',
                'ActionUrl': 'https://actions.globus.org/search/ingest',
                'ExceptionOnActionFailure': False,
                'InputPath': '$.PublishGatherMetadata.details.result[0].search',
                'ResultPath': '$.PublishIngest',
                'WaitTime': 300,
                'End': True
            },
        }
    }

    required_input = [
        'pilot',
        'funcx_endpoint_non_compute',
    ]

    flow_input = {

    }

    funcx_functions = [
        publish_gather_metadata,
    ]



from gladier import GladierBaseTool, generate_flow_definition


def asymmetric_encrypt(**data):
    import os
    from cryptography.hazmat.primitives.serialization import \
        load_ssh_public_key
    from cryptography.hazmat.primitives.asymmetric import padding
    from cryptography.hazmat.primitives import hashes

    public_file = data['public_key_path']
    if '~' in public_file:
        public_file = os.path.expanduser(public_file)

    with open(public_file, 'rb') as pub_file:
        public_key = load_ssh_public_key(pub_file.read())

    infile = data['asym_encrypt_file']
    if '~' in infile:
        infile = os.path.expanduser(infile)

    with open(infile, 'rb') as in_file:
        message = in_file.read()

    ciphertext = public_key.encrypt(
        message,
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None
        )
    )

    with open(f'{infile}.rsa', 'wb') as outfile:
        outfile.write(ciphertext)

    return f'{infile}.rsa'


@generate_flow_definition
class AsymmetricEncrypt(GladierBaseTool):
    """
    The Asymmetric Encrypt tool takes in a file and the path to the RSA public
    key to perform RSA encryption on the file.
    Adds an extension (.rsa) to the name of the file.
    It has not been found to be compatible with 3rd party encryption/decryption
    tools.

    :param public_key_path: Path to the .pub file which contains
        the RSA public key. Defaults to ~/.ssh/id_rsa.pub
    :param asym_encrypt_file: File which needs to be encrypted.
    :param funcx_endpoint_compute: By default, uses the ``compute``
        funcx endpoint.
    :returns output_path: Location of the encrypted file.
    """

    funcx_functions = [asymmetric_encrypt]
    required_input = [
        'public_key_path',
        'asym_encrypt_file',
        'funcx_endpoint_compute'
    ]
    flow_input = {
        'public_key_path': '~/.ssh/id_rsa.pub'
    }


from unittest.mock import Mock
import tarfile, os
import pytest
from gladier_tools.posix.untar import untar_file

@pytest.fixture
def mock_isfile(monkeypatch):
    """Mock isfile. Returns False by default"""
    monkeypatch.setattr(os.path, 'isfile', Mock(return_value=False))
    return os.path.isfile

def test_no_file(monkeypatch, mock_isfile):
    with pytest.raises(NameError):
        untar_file()

def test_mkdir(monkeypatch, mock_isfile, mock_tar):
    mock_isfile.return_value= True
    monkeypatch.setattr(os.path, 'exists', Mock(return_value=False))
    mock_mkdir= Mock()
    monkeypatch.setattr(os, 'makedirs', mock_mkdir)
    mock_tf, mock_context_manager= mock_tar
    untar_file()
    assert mock_mkdir.called
    assert mock_tf.called
    assert mock_context_manager.extractall.called
    





from gladier import GladierBaseTool


def https_download_file(**data):
    """Download a file from HTTPS server"""
    import os
    import requests

    ##minimal data inputs payload
    server_url = data.get('server_url', '')
    file_name = data.get('file_name', '')
    file_path = data.get('file_path', '')
    headers = data.get('headers', '')
    ##extra data inputs payload
    ##
    ##

    if server_url==None:
        raise(NameError('No `server URL` specified'))
    
    if file_name==None:
        raise(NameError('No `file_name` specified'))

    file_url = os.path.join(server_url,file_name)

    if not os.path.exists(file_path):
        os.mkdir(file_path)

    full_name = os.path.join(file_path,file_name)
    
    if not os.path.isfile(full_name):
        r = requests.get(file_url, headers=headers)
        if not r.status_code==200:  
            raise r.raise_for_status()
        open(full_name , 'wb').write(r.content)

    return full_name


class HttpsDownloadFile(GladierBaseTool):

    flow_definition = {
        'Comment': 'Downloads one file through https',
        'StartAt': 'httpDownloadFile',
        'States': {
            'httpDownloadFile': {
                'ActionUrl': 'https://api.funcx.org/automate',
                'Comment': None,
                'ExceptionOnActionFailure': True,
                'Parameters': {
                    'tasks': [
                        {
                            'endpoint.$': '$.input.funcx_endpoint_non_compute',
                            'function.$': '$.input.https_download_file_funcx_id',
                            'payload.$': '$.input'
                        }
                    ]
                },
                'ResultPath': '$.httpDownloadFile',
                'Type': 'Action',
                'WaitTime': 300,
                'End': True,
            },
        }
    }

    funcx_functions = [https_download_file]
    flow_input = {
        'headers':'',
        }
    required_input = [
        'server_url',
        'file_name',
        'file_path',
        'headers',
        'funcx_endpoint_non_compute'
        ]



from gladier import GladierBaseTool, generate_flow_definition


def decrypt(**data):

    import os
    from cryptography.fernet import Fernet
    from cryptography.hazmat.primitives import hashes
    from cryptography.hazmat.primitives.kdf.concatkdf import ConcatKDFHash
    import base64

    password = bytes(data['decrypt_key'], 'utf-8')
    ckdf = ConcatKDFHash(algorithm=hashes.SHA256(), length=32,
                         otherinfo=None, )
    key = base64.urlsafe_b64encode(ckdf.derive(password))
    fernet = Fernet(key)

    infile = data['decrypt_input']
    if '~' in infile:
        infile = os.path.expanduser(infile)

    outfile = data.get('decrypt_output', infile[:len(infile)-4])
    print(outfile)
    if '~' in outfile:
        outfile = os.path.expanduser(outfile)

    with open(infile, 'rb') as file:
        encrypted = file.read()
    decrypted = fernet.decrypt(encrypted)
    with open(outfile, 'wb') as out:
        out.write(decrypted)
    return outfile


@generate_flow_definition
class Decrypt(GladierBaseTool):
    """
    Decrypt tool takes in an encrypted file and a password to perform
    decryption on the file.
    The decryption only works on files that have been encrypted by the
    Gladier Encrypt tool.
    It has not been found to be compatible with 3rd party
    encryption/decryption tools.

    :param decrypt_input: Path to the file which needs to be decrypted.
    :param decrypt_key: Symmetric key or "password" which will be used
        to decrypt the encrypted file. Must be the same key that was
        used during encryption.
    :param decrypt_output: (optional) The full path to the decrypted file.
        If not provided, the decrypted file will have the same name as the input
        file, with the last 4 characters truncated(assuming it was a .aes file).
    :param funcx_endpoint_compute: By default, uses the ``compute`` funcx
        endpoint.
    :returns output_path: Location of the decrypted file.
    """

    funcx_functions = [decrypt]
    required_input = [
        'decrypt_input',
        'decrypt_key',
        'funcx_endpoint_compute'
    ]



import requests
from fair_research_login import NativeClient  # pip install fair-research-login

url = 'https://develop.concierge.nick.globuscs.info/api/manifest/remote_file_manifest/'
scope = 'https://auth.globus.org/scopes/524361f2-e4a9-4bd0-a3a6-03e365cac8a9/concierge'

manifest = {
    'remote_file_manifest': [
        {
            'url': 'globus://ddb59aef-6d04-11e5-ba46-22000b92c6ec/share/godata/file1.txt',
            'filename': 'file1.txt',
            'length': 4,
            'sha256': 'c8b08da5ce60398e1f19af0e5dccc744df274b826abe585eaba68c525434806',
        },
        {
            'url': 'globus://ddb59aef-6d04-11e5-ba46-22000b92c6ec/share/godata/file2.txt',
            'filename': 'file2.txt',
            'length': 4,
            'sha256': 'dd8ed44a83ff94d557f9fd0412ed5a8cbca69ea04922d88c01184a07300a5a',
        },
        {
            'url': 'globus://ddb59aef-6d04-11e5-ba46-22000b92c6ec/share/godata/file3.txt',
            'filename': 'file3.txt',
            'length': 6,
            'sha256': 'f6936912184481f5edd4c304ce27c5a1a827804fc7f329f43d273b8621870776',
        }
    ]
}

client = NativeClient(client_id='e6c75d97-532a-4c88-b031-8584a319fa3e')
client.login(requested_scopes=scope)
headers = {'Authorization': f'Bearer {client.load_tokens_by_scope(requested_scopes=scope)[scope]["access_token"]}'}
r = requests.post(url, headers=headers, json=manifest)
r.raise_for_status()
print(f'Created manifest: {r.json()["id"]}')





from gladier.base import GladierBaseTool
from .funcx_functions import manifest_to_funcx_tasks, manifest_dummy_compute_callback


class ManifestTransfer(GladierBaseTool):

    flow_definition = {
        'Comment': 'Hello Gladier Automate Flow',
        'StartAt': 'ManifestTutorial',
        'States': {
            'ManifestTutorial': {
                'Comment': 'Manifest Transfer Globus Tutorial Endpoint 1 /share/godata/ files',
                'Type': 'Action',
                'ActionUrl': 'https://develop.concierge.nick.globuscs.info/api/automate/transfer',
                'ActionScope': 'https://auth.globus.org/scopes/524361f2-e4a9-4bd0-a3a6-03e365cac8a9/concierge',
                'Parameters': {
                    'manifest_id.$': '$.input.manifest_id',
                    'destination.$': '$.input.manifest_destination',
                },
                'ResultPath': '$.ManifestTutorialResult',
                'WaitTime': 300,
                'End': True,
            },
        }
    }

    required_input = [
        'manifest_id',
        'manifest_destination'
    ]

    flow_input = {
        # Contains tutorial files on /share/godata/ for Globus Tutorial Endpoint 1
        'manifest_id': '22ea05b3-a708-4524-b2c7-b3a635ffb1c3',
        # By default, this will transfer to Globus Tutorial Endpoint 2
        'manifest_destination': 'globus://ddb59af0-6d04-11e5-ba46-22000b92c6ec/~/'
    }

    # Flow uses no FuncX functions
    funcx_functions = []


class ManifestToFuncXTasks(GladierBaseTool):
    flow_definition = {
        'Comment': 'Turn a set of files or directories into funcx tasks',
        'StartAt': 'CreateFuncXTasks',
        'States': {
            'CreateFuncXTasks': {
                'Comment': 'Fetch a manifest, extract a list of files to process, and create a funcx '
                           'task list',
                'Type': 'Action',
                'ActionUrl': 'https://api.funcx.org/automate',
                'ActionScope': 'https://auth.globus.org/scopes/facd7ccc-c5f4-42aa-916b-a0e270e2c2a9/automate2',
                'Parameters': {
                    'tasks': [{
                        'endpoint.$': '$.input.funcx_endpoint_non_compute',
                        'func.$': f'$.input.manifest_to_funcx_tasks_funcx_id',
                        'payload.$': '$.input'
                    }]
                },
                'ResultPath': '$.CreateFuncXTasksOutput',
                'WaitTime': 300,
                'Next': 'RunFuncXTasks',
            },
            'RunFuncXTasks': {
                'Comment': 'Run a funcx task on each file or directory',
                'Type': 'Action',
                'ActionUrl': 'https://api.funcx.org/automate',
                'ActionScope': 'https://auth.globus.org/scopes/facd7ccc-c5f4-42aa-916b-a0e270e2c2a9/automate2',
                # Fetch the input generated by the previous task
                'InputPath': '$.CreateFuncXTasksOutput.details.result',
                'ResultPath': '$.ManifestFuncXTasksOutput',
                'WaitTime': 300,
                'End': True,
            },
        }
    }

    required_input = [
        'funcx_endpoint_non_compute',
        'manifest_to_funcx_tasks_manifest_id',

        'manifest_to_funcx_tasks_funcx_endpoint_compute',
        # 'manifest_to_funcx_tasks_callback_funcx_id'
    ]

    flow_input = {
        # Contains tutorial files on /share/godata/ for Globus Tutorial Endpoint 1
        'manifest_to_funcx_tasks_manifest_id': '22ea05b3-a708-4524-b2c7-b3a635ffb1c3',
        'manifest_to_funcx_tasks_suffix': '',
        'manifest_to_funcx_tasks_use_dirs': False,
    }

    # Flow uses no FuncX functions
    funcx_functions = [
        manifest_to_funcx_tasks,
        manifest_dummy_compute_callback,
    ]



from gladier import GladierBaseTool


def tar(**data):
    import os
    import tarfile
    import pathlib

    tar_input = pathlib.Path(data['tar_input']).expanduser()
    tar_output = data.get('tar_output', f'{tar_input}.tgz')

    # Move to the parent directory before archiving. This ensures the
    # archive does not contain unnecessary path hierarchy.
    os.chdir(tar_input.parent)
    with tarfile.open(tar_output, 'w:gz') as tf:
        tf.add(tar_input.name)

    return tar_output


class Tar(GladierBaseTool):
    """
    The Tar tool makes it possible to create Tar archives from folders.

    :param tar_input: Input directory to archive.
    :param tar_output: (optional) output file to save the new archive. Defaults to the original
                       input file with an extension (myfile.tgz) if not given.
    :param funcx_endpoint_compute: By default, uses the ``compute`` funcx endpoint.
    :returns path: The name of the newly created archive.
    """

    # Custom flow definition to set 'ExceptionOnActionFailure' to True. We don't
    # want a transfer to start if tarring fails
    flow_definition = {
        'Comment': 'Flow with states: Tar a given folder',
        'StartAt': 'Tar',
        'States': {
            'Tar': {
                'ActionUrl': 'https://automate.funcx.org',
                'ActionScope': 'https://auth.globus.org/scopes/'
                               'b3db7e59-a6f1-4947-95c2-59d6b7a70f8c/action_all',
                'Comment': None,
                'ExceptionOnActionFailure': True,
                'Parameters': {
                    'tasks': [
                        {
                            'endpoint.$': '$.input.funcx_endpoint_compute',
                            'function.$': '$.input.tar_funcx_id',
                            'payload.$': '$.input'
                        }
                    ]
                },
                'ResultPath': '$.Tar',
                'Type': 'Action',
                'WaitTime': 300,
                'End': True,
            },
        }
    }

    funcx_functions = [tar]
    required_input = [
        'tar_input',
        'funcx_endpoint_compute',
    ]





def manifest_to_funcx_tasks(data):
    """Create funcx execution tasks given a manifest"""
    import os
    import requests
    from urllib.parse import urlparse
    required = [
        'manifest_to_funcx_tasks_manifest_id',
        'manifest_to_funcx_tasks_funcx_endpoint_compute',
        'manifest_to_funcx_tasks_callback_funcx_id',
    ]
    missing = [r for r in required if r not in data.keys()]
    if any(missing):
        raise ValueError(f'{missing} inputs MUST be included')

    url = (f'https://develop.concierge.nick.globuscs.info/api/manifest/'
           f'{data["manifest_to_funcx_tasks_manifest_id"]}/remote_file_manifest/')
    response = requests.get(url).json()
    paths = [m['url'] for m in response['remote_file_manifest']]
    if data.get('manifest_to_funcx_tasks_suffixes'):
        paths = [p for p in paths
                 if any((p.endswith(s) for s in data['manifest_to_funcx_tasks_suffixes']))]
    if data.get('manifest_to_funcx_tasks_use_dirs') is True:
        paths = list({os.path.dirname(p) for p in paths})
    tasks = []
    for path in paths:
        task = dict(
            endpoint=data['manifest_to_funcx_tasks_funcx_endpoint_compute'],
            func=data['manifest_to_funcx_tasks_callback_funcx_id'],
        )
        purl = urlparse(path)
        task['payload'] = data.get('manifest_to_funcx_tasks_payload', {})
        task['payload']['protocol'] = purl.scheme
        task['payload']['host'] = purl.netloc
        task['payload']['path'] = purl.path
        tasks.append(task)
    return {'tasks': tasks}


def manifest_dummy_compute_callback(data):
    return f'Dummy callback on manifest path: {data["path"]}'



from gladier import GladierBaseTool, generate_flow_definition


def encrypt(**data):

    import os
    from cryptography.fernet import Fernet
    from cryptography.hazmat.primitives import hashes
    from cryptography.hazmat.primitives.kdf.concatkdf import ConcatKDFHash
    import base64

    password = bytes(data['encrypt_key'], 'utf-8')
    ckdf = ConcatKDFHash(algorithm=hashes.SHA256(), length=32,
                         otherinfo=None, )
    key = base64.urlsafe_b64encode(ckdf.derive(password))
    fernet = Fernet(key)

    infile = data['encrypt_input']

    if '~' in infile:
        infile = os.path.expanduser(infile)

    outfile = infile+".aes"

    if os.path.isdir(infile):
        raise Exception("Please input the path to a file \
            or a tarred directory.")

    # opening the original file to encrypt
    with open(infile, 'rb') as file:
        original = file.read()

    # encrypting the file
    encrypted = fernet.encrypt(original)

    # opening the file in write mode and
    # writing the encrypted data
    with open(outfile, 'wb+') as encrypted_file:
        encrypted_file.write(encrypted)

    return outfile


@generate_flow_definition
class Encrypt(GladierBaseTool):
    """
    The Encrypt tool takes in a file and a password to perform
    128-bit AES symmetric key encryption on the file.
    The original contents of the file are overwritten with the encrypted text.
    Adds an extension (.aes) to the name of the file.
    It has not been found to be compatible with 3rd party encryption/decryption
    tools.

    :param encrypt_input: Path to the file which needs to be encrypted.
    :param encrypt_key: Symmetric key or "password" which can be used to
        decrypt the encrypted file.
    :param funcx_endpoint_compute: By default, uses the ``compute``
        funcx endpoint.
    :returns output_path: Location of the encrypted file.
    """

    funcx_functions = [encrypt]
    required_input = [
        'encrypt_input',
        'encrypt_key',
        'funcx_endpoint_compute'
    ]



import pytest
import tarfile
from unittest.mock import Mock

@pytest.fixture
def mock_tar(monkeypatch):
    mock_tf = Mock()
    mock_open = Mock()
    mock_open.return_value.__enter__ = Mock(return_value=mock_tf)
    mock_open.return_value.__exit__ = Mock(return_value=None)
    monkeypatch.setattr(tarfile, 'open', mock_open)
    return mock_open, mock_tf



from gladier import GladierBaseTool, generate_flow_definition


def asymmetric_decrypt(**data):
    import os
    from cryptography.hazmat.primitives.serialization import \
        load_ssh_private_key
    from cryptography.hazmat.primitives.asymmetric import padding
    from cryptography.hazmat.primitives import hashes

    private_file = data['private_key_path']
    if '~' in private_file:
        private_file = os.path.expanduser(private_file)

    password = data.get('asym_decrypt_password', None)

    with open(private_file, 'rb') as pri_file:
        private_key = load_ssh_private_key(pri_file.read(), password)

    infile = data['asym_decrypt_file']
    if '~' in infile:
        infile = os.path.expanduser(infile)

    with open(infile, 'rb') as in_file:
        ciphertext = in_file.read()

    plaintext = private_key.decrypt(
        ciphertext,
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None
        )
    )

    output_file = data.get('output_file', infile[:len(infile)-4])
    if '~' in output_file:
        output_file = os.path.expanduser(infile)

    with open(output_file, 'wb') as outfile:
        outfile.write(plaintext)

    return output_file


@generate_flow_definition
class AsymmetricDecrypt(GladierBaseTool):
    """
    The Asymmetric Decrypt tool takes in a file encrypted by the asymmetric
    encryption tool and  the path to the RSA public key to perform decryption
    on the file.
    The output file can be passed in as a flow_input. If no output file
    passed, the last 4 characters (.rsa) of the input file are removed.
    It has not been found to be compatible with 3rd party encryption/decryption
    tools.

    :param private_key_path: Path to the id_rsa file which contains
        the RSA private key. Defaults to ~/.ssh/id_rsa if not passed in.
    :param asym_decrypt_file: File which needs to be decrypted.
    :param funcx_endpoint_compute: By default, uses the ``compute``
        funcx endpoint.
    :param asym_decrypt_password: (Optional) If the private file is password
        protected, pass it in through this argument.
    :param output_file: (Optional) Path to the output file which holds the
        decrypted contents.
    :returns output_path: Location of the decrypted file.
    """

    funcx_functions = [asymmetric_decrypt]
    required_input = [
        'private_key_path',
        'asym_decrypt_file',
        'funcx_endpoint_compute'
    ]

    flow_input = {
        'private_key_path': '~/.ssh/id_rsa'
    }



from gladier import GladierBaseTool


class Transfer(GladierBaseTool):
    """
    Transfer is a simple single-state flow with no FuncX Functions, which talks directly
    to the Transfer Action Provider. It transfers only a single file or directory.

    :param transfer_source_endpoint_id: Globus Source Endpoint UUID
    :param transfer_destination_endpoint_id: Globus Destination Endpoint UUID
    :param transfer_source_path: Globus Source Path
    :param transfer_destination_path: Globus Destination Path
    :param transfer_recursive: True if this is a directory, false otherwise.
    """

    """
    The Transfer tool makes it possible to transfer data (files/folders) between globus endpoints.

    :param transfer_source_path: Path to the data that needs to be transferred in the source endpoint.
    :param transfer_destination_path: Path where the data will be transferred to in the destination endpoint.
    :param transfer_source_endpoint_id: Globus ID of the endpoint from which data will be transferred.
    :param transfer_destination_endpoint_id: Globus ID of the endpoint to which data will be transferred.
    :param transfer_recursive: Set to True if all the contents within a directory need to be transferred recursively, False otherwise.
    :returns success: Whether or not the transfer was successful.
    """

    flow_definition = {
        'Comment': 'Transfer a file or directory in Globus',
        'StartAt': 'Transfer',
        'States': {
            'Transfer': {
                'Comment': 'Transfer a file or directory in Globus',
                'Type': 'Action',
                'ActionUrl': 'https://actions.automate.globus.org/transfer/transfer',
                'Parameters': {
                    'source_endpoint_id.$': '$.input.transfer_source_endpoint_id',
                    'destination_endpoint_id.$': '$.input.transfer_destination_endpoint_id',
                    'transfer_items': [
                        {
                            'source_path.$': '$.input.transfer_source_path',
                            'destination_path.$': '$.input.transfer_destination_path',
                            'recursive.$': '$.input.transfer_recursive',
                        }
                    ]
                },
                'ResultPath': '$.Transfer',
                'WaitTime': 600,
                'End': True
            },
        }
    }

    flow_input = {
        'transfer_sync_level': 'checksum'
    }
    required_input = [
        'transfer_source_path',
        'transfer_destination_path',
        'transfer_source_endpoint_id',
        'transfer_destination_endpoint_id',
        'transfer_recursive',
    ]
